==> server-api-impl-10.2.13-sources.jar: com/xebialabs/deployit/core/rest/api/reports/mapper/ReportToCsvMapper.java <==
package com.xebialabs.deployit.core.rest.api.reports.mapper;

import java.util.Collection;
import java.util.List;
import java.util.Map;
import java.util.TreeMap;
import java.util.function.Function;
import java.util.stream.Collectors;
import java.util.stream.Stream;
import java.util.stream.StreamSupport;
import org.checkerframework.checker.nullness.qual.Nullable;
import com.google.common.base.Joiner;

import com.xebialabs.deployit.core.api.dto.ReportLine;

public class ReportToCsvMapper {
    private RowMapper rowMapper;

    public ReportToCsvMapper(final Stream<ReportLine> reportLines) {
        rowMapper = new ImplicitRowMapper(reportLines);
    }

    public ReportToCsvMapper(final Collection<ReportLine> reportLines) {
        this(reportLines.stream());
    }

    public ReportToCsvMapper(final Stream<ReportLine> reportLines, final Collection<String> fields) {
        if (fields == null) {
            rowMapper = new ImplicitRowMapper(reportLines);
        } else {
            rowMapper = new ExplicitRowMapper(reportLines, sortFields(fields));
        }
    }

    private List<String> sortFields(final Collection<String> fields) {
        return fields.stream().sorted().collect(Collectors.toList());
    }

    public Stream<String> getCsv() {
        return rowMapper.getRows();
    }

    private abstract static class RowMapper {
        protected static final Joiner LINE_JOINER = Joiner.on("\n");
        protected static final Joiner FIELD_JOINER = Joiner.on(",");
        protected static final Function<Object, @Nullable String> QUOTE = (final Object input) -> String.format("\"%s\"", input);

        protected final Stream<ReportLine> reportLines;

        public abstract Stream<String> getRows();

        RowMapper(final Stream<ReportLine> reportLines) {
            this.reportLines = reportLines;
        }

        protected String line(final Iterable<?> fields) {
            return FIELD_JOINER.join(StreamSupport.stream(fields.spliterator(), false).map(QUOTE).collect(Collectors.toList()));
        }
    }

    private static class ImplicitRowMapper extends RowMapper {
        private boolean first = true;

        ImplicitRowMapper(final Stream<ReportLine> reportLines) {
            super(reportLines);
        }

        @Override
        public Stream<String> getRows() {
            return reportLines.map(this::getLine);
        }

        private String getLine(final ReportLine reportLine) {
            TreeMap<String, Object> sorted = sorted(reportLine);
            if (first) {
                first = false;
                return LINE_JOINER.join(
                        line(sorted.keySet()),
                        line(sorted.values())
                );
            } else {
                return line(sorted.values());
            }

        }

        private static TreeMap<String, Object> sorted(final ReportLine line) {
            return new TreeMap<>(line.getValues());
        }
    }

    private static class ExplicitRowMapper extends RowMapper {
        private List<String> fields;

        ExplicitRowMapper(final Stream<ReportLine> reportLines, final List<String> fields) {
            super(reportLines);
            this.fields = fields;
        }

        @Override
        public Stream<String> getRows() {
            return Stream.concat(Stream.of(getHeader()), reportLines.map(this::getLine));
        }

        private String getHeader() {
            return line(fields);
        }

        private String getLine(final ReportLine reportLine) {
            Map<String, Object> values = reportLine.getValues();
            List<Object> sortedValues = this.fields.stream().map(values::get).collect(Collectors.toList());
            return line(sortedValues);
        }


    }
}

==> server-api-impl-24.1.10-sources.jar: com/xebialabs/deployit/core/rest/api/reports/mapper/ReportToCsvMapper.java <==
package com.xebialabs.deployit.core.rest.api.reports.mapper;

import ai.digital.deploy.sql.model.ReportLine;
import com.google.common.base.Joiner;
import org.checkerframework.checker.nullness.qual.Nullable;

import java.util.Collection;
import java.util.List;
import java.util.Map;
import java.util.TreeMap;
import java.util.function.Function;
import java.util.stream.Collectors;
import java.util.stream.Stream;
import java.util.stream.StreamSupport;

public class ReportToCsvMapper {
    private final RowMapper rowMapper;

    public ReportToCsvMapper(final Stream<ReportLine> reportLines) {
        rowMapper = new ImplicitRowMapper(reportLines);
    }

    public ReportToCsvMapper(final Collection<ReportLine> reportLines) {
        this(reportLines.stream());
    }

    public ReportToCsvMapper(final Stream<ReportLine> reportLines, final Collection<String> fields) {
        if (fields == null) {
            rowMapper = new ImplicitRowMapper(reportLines);
        } else {
            rowMapper = new ExplicitRowMapper(reportLines, sortFields(fields));
        }
    }

    private List<String> sortFields(final Collection<String> fields) {
        return fields.stream().sorted().collect(Collectors.toList());
    }

    public Stream<String> getCsv() {
        return rowMapper.getRows();
    }

    private abstract static class RowMapper {
        protected static final Joiner LINE_JOINER = Joiner.on("\n");
        protected static final Joiner FIELD_JOINER = Joiner.on(",");
        protected static final Function<Object, @Nullable String> QUOTE = (final Object input) -> String.format("\"%s\"", input);

        protected final Stream<ReportLine> reportLines;

        public abstract Stream<String> getRows();

        RowMapper(final Stream<ReportLine> reportLines) {
            this.reportLines = reportLines;
        }

        protected String line(final Iterable<?> fields) {
            return FIELD_JOINER.join(StreamSupport.stream(fields.spliterator(), false).map(QUOTE).collect(Collectors.toList()));
        }
    }

    private static class ImplicitRowMapper extends RowMapper {
        private boolean first = true;

        ImplicitRowMapper(final Stream<ReportLine> reportLines) {
            super(reportLines);
        }

        @Override
        public Stream<String> getRows() {
            return reportLines.map(this::getLine);
        }

        private String getLine(final ReportLine reportLine) {
            TreeMap<String, Object> sorted = sorted(reportLine);
            if (first) {
                first = false;
                return LINE_JOINER.join(
                        line(sorted.keySet()),
                        line(sorted.values())
                );
            } else {
                return line(sorted.values());
            }

        }

        private static TreeMap<String, Object> sorted(final ReportLine line) {
            return new TreeMap<>(line.getValues());
        }
    }

    private static class ExplicitRowMapper extends RowMapper {
        private final List<String> fields;

        ExplicitRowMapper(final Stream<ReportLine> reportLines, final List<String> fields) {
            super(reportLines);
            this.fields = fields;
        }

        @Override
        public Stream<String> getRows() {
            return Stream.concat(Stream.of(getHeader()), reportLines.map(this::getLine));
        }

        private String getHeader() {
            return line(fields);
        }

        private String getLine(final ReportLine reportLine) {
            Map<String, Object> values = reportLine.getValues();
            List<Object> sortedValues = this.fields.stream().map(values::get).collect(Collectors.toList());
            return line(sortedValues);
        }


    }
}
